<?php
/**
 * Zarafa Z-Merge SugarCRM Connector
 *
 * Copyright (C) 2005 - 2008  Zarafa B.V.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
 * and under the terms of the GNU General Public License, version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU (Affero) General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * and the GNU General Public License along with this program.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * Created on 22.04.2008 by Sebastian Kummer & Manfred Kutas
 *
 * @package Z-Merge
 * @subpackage SugarCRM-Connector
 */



/**
* checkConfig() function.
* tried to access the database. If not available, it will try to create the tables itself
*
* @return boolean
*/
function checkDatabase() {
        global $db, $db_creation;
        if (!$db) $db = & DBManager :: getInstance();
        if ($db->tableExists("zmerge")) {
            return true;
        }

        // create db
        foreach ($db_creation as $dbc) {
            $db->query($dbc);
        }

        // check again!
        if ($db->tableExists("zmerge")) {
            return true;
        }
        return false;
}


/**
* getUTC() function.
* converts SugarCRM-dates in UTC-Timestamps incl local time settings
*
* @param String $timestring    Timestring in a SugarCRM own format
* @return long
*/
function getUTC($timestring) {
    if(preg_match("/(\d{4})-(\d{2})-(\d{2})\s(\d{2}):(\d{2})/", $timestring, $dt)) {
        return gmmktime($dt[4], $dt[5], 0, $dt[2],$dt[3], $dt[1]);
    }
    return 0;
}



/**
 * Retrieve field data for a provided SugarBean. (Copyright by SugarCRM)
 *
 * @param SugarBean $value -- The bean to retrieve the field information for.
 * @return Array -- 'field'=>   'name'     -- the name of the field
 *                              'type'     -- the data type of the field
 *                              'label'    -- the translation key for the label of the field
 *                              'required' -- Is the field required?
 *                              'options'  -- Possible values for a drop down field
 */
function get_field_list(&$value){
    $list = array();
    if(!empty($value->field_defs)){
        foreach($value->field_defs as $var){
            if(isset($var['source']) && $var['source'] != 'db' && (!isset($var['type'])|| $var['type'] != 'relate'))continue;
            $required = 0;
            $options_dom = array();
            $options_ret = array();
            // Apparently the only purpose of this check is to make sure we only return fields
            //   when we've read a record.  Otherwise this function is identical to get_module_field_list
            if(isset($value->required_fields) && key_exists($var['name'], $value->required_fields)){
                $required = 1;
            }
            if(isset($var['options'])){
                $options_dom = translate($var['options'], $value->module_dir);
                if(!is_array($options_dom)) $options_dom = array();
                foreach($options_dom as $key=>$oneOption)
                    $options_ret[] = get_name_value($key,$oneOption);
            }
            if(!empty($var['custom_type'])){

                $var['type'] = $var['custom_type'];
                //don't support multi enum through portal
                if(strcmp($var['type'],  'multienum') ==0)continue;
             }
            if(!empty($var['dbType']) && $var['type'] == 'bool') {
                $options_ret[] = get_name_value('type', $var['dbType']);
            }
            $list[$var['name']] = array('name'=>$var['name'],
                                        'type'=>$var['type'],
                                        'label'=>translate($var['vname'], $value->module_dir),
                                        'required'=>$required,
                                        'options'=>$options_ret );
        }
    }
    return $list;
}


/**
* get_name_value() function.
* returns an name->value array for the fieldname and its value
*
* @param String $field    a fieldname
* @param String $value    the value
* @return long
*/
function get_name_value($field,$value){
    return array('name'=>$field, 'value'=>$value);
}


/**
* getUserName() function.
* gets the human readable username for an user id
*
* @param String $user_id    internal id of the user
* @return String|null
*/
function getUserName($user_id) {
    global $db;
    $query = "SELECT user_name from users where id='$user_id' AND deleted=0";
    $result = $db->query($query, false, "Error retrieving user name from ID: ");
    $row = $db->fetchByAssoc($result);
    return $row['user_name'];
}


/**
* getUserId() function.
* gets the user id from the human readable username
*
* @param String $user_name    username of the user
* @return String|null
*/
function getUserId($user_name) {
    global $db;
    $query = "SELECT id from users where user_name='$user_name' AND deleted=0";
    $result = $db->query($query, false, "Error retrieving user ID: ");
    $row = $db->fetchByAssoc($result);
    return $row['id'];
}


/**
* parseAddr() function.
* parses MimeDecode address objects back to a simple "," separated string
*
* @param String $ad    address object
* @return String
*/
function parseAddr($ad) {
    $addr_string = "";
    if (isset($ad) && is_array($ad)) {
        foreach($ad as $addr) {
            if ($addr_string) $addr_string .= ",";
            if (isset($addr->mailbox) && isset($addr->host))
                $addr_string .= $addr->mailbox . "@" . $addr->host;
        }
    }
    return $addr_string;
}

/**
* parseAddrName() function.
* parses MimeDecode address objects back to a simple "," separated string with full names
*
* @param String $ad    address object
* @return String
*/
function parseAddrName($ad) {
    $addr_string = "";
    if (isset($ad) && is_array($ad)) {
        foreach($ad as $addr) {
            if ($addr_string) $addr_string .= ",";
            if (isset($addr->personal) && isset($addr->mailbox) && isset($addr->host))
                $addr_string .= $addr->personal . " <" . $addr->mailbox . "@" . $addr->host . ">";
            else if (isset($addr->mailbox) && isset($addr->host))
                $addr_string .= $addr->mailbox . "@" . $addr->host;
        }
    }
    return $addr_string;
}


/**
* getBodyRecursive() function.
* Get all parts in the message with specified type and concatenate them together, unless the
* Content-Disposition is 'attachment', in which case the text is apparently an attachment
*
* @param String $ad    address object
* @return String
*/
function getBodyRecursive($message, $subtype, &$body) {
    if(!isset($message->ctype_primary)) return;
    if(strcasecmp($message->ctype_primary,"text")==0 && strcasecmp($message->ctype_secondary,$subtype)==0 && isset($message->body))
        $body .= $message->body;

    if(strcasecmp($message->ctype_primary,"multipart")==0 && isset($message->parts) && is_array($message->parts)) {
        foreach($message->parts as $part) {
            if(!isset($part->disposition) || strcasecmp($part->disposition,"attachment"))  {
                getBodyRecursive($part, $subtype, $body);
            }
        }
    }
}


/**
* isCompatibleSugarVersion() function.
* Checks if the supplied SugarCRM version is compatible with the connector
*
* @param String $sugar_version    the sugar version to be tested
* @return boolean
*/
function isCompatibleSugarVersion($sugar_version) {
    global $sup_versions;

    if (SG_IGNORE_VERSION === true)
        return true;

    foreach($sup_versions as $v ){
        if(preg_match("/$v/", $sugar_version))
            return true;
    }

    return false;
}

?>